/*
 * parmes.cpp
 *
 *  Created on: 2010/09/21
 *      Author: shu
 */

#include "aligner_parameters.h"
#include "sequence_type.h"
#include "protein_type.h"
#include "dna_type.h"
#include "encoder.h"
#include "score_matrix.h"
#include "score_matrix_reader.h"
#include "logger.h"
#include <stdexcept>
#include <string>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <typeinfo>
#include <getopt.h>

using namespace std;

AlignerParameters::AlignerParameters() :
  number_thread_(1), queries_filename_(""), db_filename_(""), output_filename_(""), query_type_(new ProteinType()),
  aligned_type_(new ProteinType()), search_length_(8), threshold_(30), search_cutoff_(1),
  max_number_hits_(1 << 21), max_queries_length_(1 << 27),
  max_number_alignments_(1), max_number_one_subject_alignments_(1), max_gapped_size_(50),
  open_gap_(11), extend_gap_(1) ,  ungapped_extend_normalized_cutoff_(7.0), gapped_extend_normalized_cutoff_(15.0),
  final_gapped_extend_normalized_cutoff_(25.0), gapped_extend_normalized_trigger_(22.0),
  kNextQueryIdIncrement_(1), kNextAlignmentIdIncrement_(1), entropy_threshold_for_reordering_(3.0), 
  entropy_threshold_for_filtering_(0.0), device_id_(kCpuId)
{
  SetScoringParameters(*aligned_type_, string(""), open_gap_, extend_gap_);  
}

AlignerParameters::~AlignerParameters() {}


void AlignerParameters::SetParameters(int argc, char* argv[]) {
  string score_matrix_name = "";
  int   c;

  extern char *optarg;
  extern int  optind;
  optind = 1;

  while ((c = getopt(argc, argv, "a:b:d:D:E:f:G:i:l:L:m:M:n:o:q:r:s:S:t:T:v:w:")) != -1){
    switch (c){
    case 'a':
      number_thread_ = atoi(optarg);
      break;

    case 'b':
      max_number_alignments_ = atoi(optarg);
      break;

    case 'd':
      db_filename_ = optarg;
      break;

    case 'D':
      throw invalid_argument(string("can't use GPU"));
      break;

    case 'E':
      extend_gap_ = atoi(optarg);
      break;

    case 'f':
      entropy_threshold_for_filtering_ = atoi(optarg);
      break;

    case 'G':
      open_gap_ = atoi(optarg);
      break;

    case 'i':
      queries_filename_ = optarg;
      break;

    case 'l':
      max_queries_length_ = atoi(optarg);
      break;

    case 'L':
      max_number_hits_ = atoi(optarg);
      break;

    case 'm':
      kNextQueryIdIncrement_ = atoi(optarg);
      break;

    case 'M':
      score_matrix_name = optarg;
      break;

    case 'n':
      kNextAlignmentIdIncrement_ = atoi(optarg);
      break;

    case 'o':
      output_filename_ = optarg;
      break;

    case 'q':
      if(optarg[0] == 'p') {
        query_type_ = SequenceTypePtr(new ProteinType());
      } else if (optarg[0] == 'd') {
        query_type_ = SequenceTypePtr(new DnaType());
      } else {
        throw invalid_argument(string("invalid option, -q is p or d"));
      }
      break;

    case 'r':
      entropy_threshold_for_reordering_ = atof(optarg);
      break;

    case 's':
      search_cutoff_ = atoi(optarg);
      break;

    case 'S':
      search_length_ = atoi(optarg);
      break;

    case 't':
      if (optarg[0] == 'p') {
        aligned_type_ = SequenceTypePtr(new ProteinType());
        /*
      } else if (optarg[0] == 'd') {
        query_type_ = SequenceTypePtr(new DnaType());
        */
      } else {
        throw invalid_argument(string("invalid option, -t is p."));
      }
      break;

    case 'T':
      threshold_ = atoi(optarg);
      break;

    case 'w':
      max_gapped_size_ = atoi(optarg);
      break;

    case 'v':
      max_number_one_subject_alignments_ = atoi(optarg);
      break;

    default:
      throw invalid_argument("\nTry `ghostx --help' for more information.");
    }
  }

  SetScoringParameters(*aligned_type_, score_matrix_name, open_gap_, extend_gap_);
}

void AlignerParameters::SetScoringParameters(SequenceType &type, string name, int open_gap, int extend_gap) {
  Encoder encoder(type);
  ScoreMatrixReader score_matrix_reader(encoder);
  if (name.length() != 0) {
    score_matrix_ = score_matrix_reader.Read(name);
  } else {
    score_matrix_ = score_matrix_reader.ReadDefault();
  }

  Statistics statistics(type);
  statistics.CalculateUngappedIdealKarlinParameters(score_matrix_, &ungapped_ideal_karlin_parameters_);
  ungapped_ideal_extend_nominal_cutoff_ = Statistics::NormalizedCutoff2NominalCutoff(ungapped_extend_normalized_cutoff_, ungapped_ideal_karlin_parameters_);
  gapped_ideal_extend_nominal_trigger_ = Statistics::Normalized2Nominal(gapped_extend_normalized_trigger_, ungapped_ideal_karlin_parameters_);
  statistics.CalculateGappedKarlinParameters(score_matrix_, open_gap, extend_gap, &gapped_karlin_parameters_);
  gapped_ideal_extend_nominal_cutoff_ = Statistics::NormalizedCutoff2NominalCutoff(gapped_extend_normalized_cutoff_, gapped_karlin_parameters_);
  final_gapped_ideal_extend_nominal_cutoff_ = Statistics::NormalizedCutoff2NominalCutoff(final_gapped_extend_normalized_cutoff_, gapped_karlin_parameters_);
  statistics.CalculateAlphaBeta(score_matrix_, open_gap, extend_gap, &alpha_, &beta_);
}
